package ISPMan::DBMan;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;

@ISA    = qw(ISPMan Exporter AutoLoader);
@EXPORT = qw(
  add_database_form
  add_vhost_database
  create_vhost_database
  drop_vhost_database
  create_database
  drop_database
  editDatabase
  getDatabases
  getDatabaseInfo
  modify_vhost_database
);
$VERSION = '0.01';

sub getDatabases {
    my $self = shift;
    my $dn   = shift;

    my $dbHash =
      $self->{'ldap'}
      ->getEntriesAsHashRef( $dn, "&(objectclass=ispmanDatabaseData)(dbname=*)",
        ["dbname"] );

    my $dbs;
    for ( keys %$dbHash ) {
        $dbs->{ $dbHash->{$_}{'dbname'} } = $_;
    }
    return $dbs;

}

sub getDatabaseInfo {
    my $self   = shift;
    my $dbname = shift;
    my $domain = shift;
    my $dn;

    if ( $dbname !~ /^dbname=/ ) {
        $dn = "dbname=$dbname, ou=databases, ispmanDomain=$domain, ";
        $dn .= $self->getConf("ldapBaseDN");
    }
    else {

        # we have been passed a dn
        $dn = $dbname;
    }

    $self->{'dbInfo'}{$domain}{$dbname} = $self->getEntryAsHashRef($dn);
    return $self->{'dbInfo'}{$domain}{$dbname};

}

sub editDatabase {
    my $self = shift;
    my $r    = shift;
    my $dn   = $r->param("dn");
    $self->{'database'} = $self->getDatabaseInfo($dn);

    my $template;

    if ( $self->isDomainEditable( $r->param("ispmanDomain") ) ) {
        $template = $self->getTemplate("databases/edit.tmpl");
    }
    else {
        $template = $self->getTemplate("databases/view.tmpl");
    }
    print $template->fill_in( PACKAGE => "ISPMan" );
}

sub modify_vhost_database {
    my $self = shift;
    my $r    = shift;
    $self->modifyAttribute( $r->param("dn"), "password",
        $r->param("password") );
    $self->editDatabase($r);
}

sub add_database_form {
    my $self     = shift;
    my $template = $self->getTemplate("databases/add.tmpl");
    print $template->fill_in( PACKAGE => "ISPMan" );
}

sub add_vhost_database {
    my ( $self, $r ) = @_;
    $self->create_vhost_database( $r->param("ispmanDomain"),
        $r->param("vhost"), $r->param("pass") );
    print "Database Scheduled for addition";
}

sub create_vhost_database {
    my $self = shift;
    my ( $domain, $vhost, $pass ) = @_;

    # this sub will only add data to the LDAP server
    # it will not really create or drop tables.

    my $data;
    $data->{'dbname'} = join '_', ( $vhost, $domain );
    $data->{'dbname'} =~ s/\./_/g;
    $data->{'dbname'} =~ s/[^a-z0-9]/_/g;

    $data->{'user'}        = join '.', ( $vhost, $domain );
    $data->{'domain'}      = $domain;
    $data->{'vhost'}       = $vhost;
    $data->{'password'}    = $pass;
    $data->{'objectclass'} = "ispmanDatabaseData";

    my $branch =
      "ou=databases, ispmanDomain=$domain, @{[$self->getConf('ldapBaseDN')]}";
    unless ( $self->entryExists($branch) ) {
        $self->addDataFromLdif( "templates/domain.databases.ldif.template",
            { 'domain' => $domain } );
    }

    my $dn =
"dbname=$data->{'dbname'}, ou=databases, ispmanDomain=$domain, @{[$self->getConf('ldapBaseDN')]}";
    unless ( $self->entryExists($dn) ) {
        print "Adding $dn\n";
        $self->addNewEntryWithData( $dn, $data );
    }
}

sub drop_vhost_database {
    my $self = shift;
    my ( $domain, $vhost ) = @_;
    my $dbname = join '_', ( $vhost, $domain );
    $dbname =~ s/\./_/g;
    my $dn =
"dbname=$dbname, ou=databases, ispmanDomain=$domain, @{[$self->getConf('ldapBaseDN')]}";
    if ( $self->entryExists($dn) ) {
        print "Dropping $dn\n";
        $self->deleteEntry($dn);
    }
}

1;

__END__

